uniform sampler2D scene;
uniform sampler2D table;
uniform sampler2D texpos;
uniform float dim;
uniform int effect;

const float width = 1024.;
const float height = 768.;

#define NOEFFECT 0
#define ASCII 1
#define SOBEL 2
#define DITHERLINES 3
#define DITHERTRIANGLES 4

float max(float a, float b)
{
	return ((a>=b)? a : b);
}

float sampleAsGrey(sampler2D tex, vec4 texCoord)
{
	vec4 col = texture2D(tex, texCoord.xy);
	float grey = 0.299 * col.r + 0.587 * col.g + 0.114 * col.b;
	return grey;
}

// ascii shader
vec4 ascii()
{
	// get color, convert it to grey value
	//vec4 col = texture2D(scene, gl_TexCoord[0].xy);
	//float grey = 0.299 * col.r + 0.587 * col.g + 0.114 * col.b;
	vec4 col;
	float grey = sampleAsGrey(scene, gl_TexCoord[0]);
	
	// get relative coordiante to the rectangle's top left corner, pixel lies in 
	vec4 pos = texture2D(texpos, gl_TexCoord[0]);
	
	// calculate texture coordinate to sample char:
    float nchar = floor(grey * 178.);
    int n = 0;
    vec2 coord = vec2(0., 0.);
	float width = 1023.;
	float height = 63.;
    
	if(nchar >= 0. && nchar < 59.)
    {
      // first row
      coord.y =  (2.+pos.g*(255./10.))*1./height;
      coord.x = ( nchar * 17. + pos.r*(255./10.) ) * 1./width;   // pixel
      
    }
	else if(nchar>=59. && nchar < 118.)
    {
      // second row
      coord.y = (17. + pos.g*(255./10.)) * 1./height;
      nchar = nchar-59.;
      coord.x = ( nchar * 17. + pos.r*(255./10.) ) * 1./width;   // pixel
    }
    else if(nchar >= 118. && nchar < 177.)
    {
      // third row
      coord.y = (31. + pos.g*(255./10.))*1./height;
      nchar = nchar-118.;
      coord.x = ( nchar * 17. + pos.r*(255./10.) ) * 1./width;   // pixel
    }
	else if(nchar >= 177. && nchar < 198.)
    {
      // forth row
      coord.y = (54.+ pos.g*(255./10.))*1./height;
      nchar = nchar-177.;
      coord.x = ( nchar * 17. + pos.r*(255./10.) ) * 1./width;   // pixel
    }
	
	coord.y = 1. - coord.y;
	
    col  = tex2D(table, coord);
    if (col.r > 0.7 && col.g > 0.7 && col.b > 0.7) 
		col = vec4(max(grey, 0.4));
    else col.rgb = vec3(0.);  // bkgd black

    return col;
}

// sobel edge detection:
vec4 sobelEdgeDetection(vec2 cTex, sampler2D sTex, float width, float height)
{
	width  = 1./(width/2.0);
	height = 1./(height/2.0);

	vec2 sampleOffsets[8] ={ 	vec2(-width,	-height), 	vec2(0.0,		-height),	vec2(width,	-height),	
								vec2(-width,	 0.0), 		vec2(width,	 0.0),
								vec2(-width,	 height),	vec2(0.0,		 height), 	vec2(width,	 height)  	
							};
	int i =0;
	vec4 c = vec4(.5);
	vec2 texCoords;
	vec4 texSamples[8];
	vec4 vertGradient;
	vec4 horzGradient;
	
	for(i =0; i < 8; i++)
	{
		texCoords = cTex + sampleOffsets[i]; 
		texSamples[i] = texture2D(sTex, texCoords);
		texSamples[i] = vec4(dot(texSamples[i], vec4(.333333)));
	}
	
	// vertical Gradient
	vertGradient = -(texSamples[0] + texSamples[5] + 2.*texSamples[3]);
	vertGradient += (texSamples[2] + texSamples[7] + 2.*texSamples[4]);
	// Horizontal Gradient
	horzGradient = -(texSamples[0] + texSamples[2] + 2.*texSamples[1]);
	horzGradient += (texSamples[5] + texSamples[7] + 2.*texSamples[6]);
	
	c = sqrt( horzGradient*horzGradient + vertGradient*vertGradient );
	return c;
}

vec4 ditherLines()
{
	// get color, convert it to grey value
	vec4 col = texture2D(scene, gl_TexCoord[0].xy);
	
	float grey = 0.299 * col.r + 0.587 * col.g + 0.114 * col.b;
	// get relative coordiante to the rectangle's top left corner, pixel lies in 
	vec4 pos = texture2D(texpos, gl_TexCoord[0].xy);
	
	// calculate texture coordinate to sample 
    vec2 coord = vec2(0., 0.);
    float t = floor( grey * 16. );	

    coord.x = ( t * 8. + pos.r * (255./10.)) * 1./128.;
    coord.y = pos.g * (255./10.)*1./12.;
	
	return texture2D(table, coord);
}

vec4 ditherTri()
{
	// get color, convert it to grey value
	vec4 col = texture2D(scene, gl_TexCoord[0].xy);
	
	float grey = 0.299 * col.r + 0.587 * col.g + 0.114 * col.b;
	// get relative coordiante to the rectangle's top left corner, pixel lies in 
	vec4 pos = texture2D(texpos, gl_TexCoord[0].xy);
	
	// calculate texture coordinate to sample 
    vec2 coord = vec2(0., 0.);
    float t = floor( grey * 60. );	

    coord.x = ( t * 8. + pos.r * (255./10.)) * 1./1024.;
    coord.y = pos.g * (255./10.)*1./12.;
	
	return texture2D(table, coord);
}

void main(void)
{
	if( effect == ASCII)
		gl_FragColor = dim * ascii();
	else if(effect == SOBEL)
		gl_FragColor = dim * sobelEdgeDetection(gl_TexCoord[0].xy, scene, 1024., 768.);
	else if( effect == DITHERLINES)
		gl_FragColor = dim * ditherLines();
	else if( effect == DITHERTRIANGLES)
		gl_FragColor = dim * ditherTri();		
	else
		gl_FragColor = dim * ascii();
}