uniform sampler2D shadowMap;
uniform int multiSampling;
uniform float dim;

varying vec4 diffuse,ambientGlobal, ambient;
varying vec3 normal,lightDir,halfVector;
varying float dist;
varying float visible;

float SelfShadowedVal = 0.75;														
float NonSelfShadowedVal = 0.65;		
vec2 texSize = vec2(1024., 768.);

vec4 calcLight()
{
	vec3 n,halfV,viewV,ldir;
	float NdotL,NdotHV;
	vec4 color = ambientGlobal;
	float att;
	
	n = normalize(normal);

	NdotL = max(dot(n,normalize(lightDir)),0.0);

	if (NdotL > 0.0) 
	{
		att = 1.0 / (gl_LightSource[0].constantAttenuation +
				gl_LightSource[0].linearAttenuation * dist +
				gl_LightSource[0].quadraticAttenuation * dist * dist);
		color += att * (diffuse * NdotL + ambient);
	
		halfV = normalize(halfVector);
		NdotHV = max(dot(n,halfV),0.0);
		color += att * gl_FrontMaterial.specular * gl_LightSource[0].specular * pow(NdotHV,gl_FrontMaterial.shininess);
	}
	
	return vec4(color.xyz, 1.);
}
	
float lookup(vec4 texCoords, vec2 offset, float Illumination)													
{																					
	if(texCoords.s<0. || texCoords.s > 1. || texCoords.t<0. || texCoords.t > 1.) 
		return 1.;
	else
	{
		vec4 shadowval = texture2D(shadowMap, texCoords.st+offset);
		if(shadowval.z  < texCoords.z) 
			return Illumination;
		else
			return 1.0; 	
	}
}

vec4 shadowTexCoords(vec4 texCoords)
{
	vec4 tc = texCoords/texCoords.w;
	return (tc+ 1.0) * 0.5; 
}
																									
float multiSampleSelfShadowed()										
{		
	vec4 texCoords = shadowTexCoords(gl_TexCoord[1]);																			
	float Illumination = visible < 0.5 ? SelfShadowedVal : NonSelfShadowedVal;		
	if(visible>0.5)															
	{																				
		float sum = 0.0;															
																				
		sum+=lookup(texCoords, 	vec2(  0., 			 -1./texSize.y), 	Illumination);				
		sum+=lookup(texCoords, 	vec2( -1./texSize.x,  0.		  ), 	Illumination);		
		sum+=lookup(texCoords, 	vec2(  1./texSize.x,  0.		  ), 	Illumination);			
		sum+=lookup(texCoords, 	vec2(  0. ,			  1./texSize.y), 	Illumination);			
		return sum*0.25;															
	}																				
	else																			
	{																				
		return Illumination;														
	}																				
}					
																																																	
void mainOneSample(void)
{
	vec4 col = calcLight();
	vec4 texCoords = shadowTexCoords(gl_TexCoord[1]);		
	float Illumination = (visible < 0.5 ) ? SelfShadowedVal : NonSelfShadowedVal;	

	if(visible < 0.5)
	{
		gl_FragColor = col*SelfShadowedVal;
	}
	else
	{
		gl_FragColor = col * lookup(texCoords, 	vec2(0., 0.), Illumination);	
	}
}

void mainMultiSample()																		
{																																	
  gl_FragColor = calcLight() * multiSampleSelfShadowed() ;											
}

void main()
{
	//mainOneSample();
	mainMultiSample();
}