uniform mat4 TransformGrid; 
uniform mat4 TransformModel;	
uniform int renderShadows; 
uniform mat4 lMVP;
uniform mat4 mMVIT;

uniform float m;
uniform float n1;
uniform float n2;
uniform float n3;

uniform float mb;
uniform float n1b;
uniform float n2b;
uniform float n3b;

varying vec4 diffuse, ambientGlobal, ambient;
varying vec3 normal,lightDir, halfVector;
varying float dist;
varying float visible;
varying float vrenderShadows;

const float PI = 3.14159265358979323846264338327950288419716939937510;

struct pd
{
   float p;
   float d; 
};

pd superformula(float pm, float pn1, float pn2, float pn3, float phi)
{
   vec2 ret;
   float a=1., b=1.;
   float t1www = cos((pm * phi) / 4.);
   float t1ww = t1www / a;
   float t1w = abs(t1ww);
   float t1 = pow(t1w, n2);
   float t2www = sin((pm * phi) / 4.);
   float t2ww = t2www / b;
   float t2w = abs(t2ww);
   float t2 = pow(t2w, pn3);
   float T = t1+t2;
   float r = pow(T, 1./pn1);
   
   if (abs(r) == 0.) 
   {
      r = 0.;
   } 
   else 
   {
      r = 1. / r;
   }
   
   float dt1 = pn2 * (t1/t1w) * sign(t1ww) * (- pm * t2www / (4. * a));
   float dt2 = pn3 * (t2/t2w) * sign(t2ww) * (  pm * t1www / (4. * b));
   float dT = dt1 + dt2;
   
   pd Out;
   Out.p = r;
   Out.d = - (1./pn1) * (r/T) * dT;
   
   return Out;
}

struct PosBiTan
{
   vec3 Pos;
   vec3 Bi;
   vec3 Tan;
};

PosBiTan superformulas3D(float pm, float pn1, float pn2, float pn3,float pmb, float pn1b, float pn2b, float pn3b, float theta, float phi)
{
  pd rt = superformula(pm, pn1, pn2, pn3, theta);
  pd rp = superformula(pmb, pn1b, pn2b, pn3b, phi);
  
  float st = sin(theta);
  float ct = cos(theta);
  float sp = sin(phi);
  float cp = cos(phi);

  PosBiTan Out;
  Out.Pos.x = rt.p * ct * rp.p * cp;
  Out.Pos.y = rt.p * st * rp.p * cp;
  Out.Pos.z = rp.p * sp;

  Out.Bi.x = (rt.d*ct-rt.p*st) * (rp.p * cp);
  Out.Bi.y = (rt.d*st+rt.p*ct) * (rp.p * cp);
  Out.Bi.z = 0.;
  Out.Tan.x = (rt.p * ct) * (rp.d*cp-rp.p*sp);
  Out.Tan.y = (rt.p * st) * (rp.d*cp-rp.p*sp);
  Out.Tan.z = (rp.d*sp+rp.p*cp);
  
  return Out;
}

void calcLight(vec4 pos, vec3 n)
{	
	vec4 ecPos;
	vec3 aux;
	
	normal = normalize(gl_NormalMatrix * n);
	
	ecPos = gl_ModelViewMatrix * pos;
	aux = vec3(gl_LightSource[0].position-ecPos);
	lightDir = normalize(aux);
	
	dist = length(aux);

	halfVector = normalize(gl_LightSource[0].halfVector.xyz);
	
	diffuse = gl_FrontMaterial.diffuse * gl_LightSource[0].diffuse;
	ambient = gl_FrontMaterial.ambient * gl_LightSource[0].ambient;
	ambientGlobal = gl_LightModel.ambient * gl_FrontMaterial.ambient;
} 

void main()
{
    vec4 Pos = TransformGrid * gl_Vertex;
    PosBiTan posbitan = superformulas3D(m, n1, n2, n3,mb, n1b, n2b, n3b, Pos.x, Pos.y);
    Pos.xyz = posbitan.Pos;
    
    gl_Position = gl_ModelViewProjectionMatrix * Pos; 
    
    gl_TexCoord[0] = gl_MultiTexCoord0;	
    
	gl_FrontColor = gl_Color;
    gl_BackColor = gl_Color; 		
 
    calcLight(Pos, Pos.xyz);    
	
	vrenderShadows = float(renderShadows);
	
	if(renderShadows == 1)
	{
		visible = 1.;
	   
		// shadow texture coordinates generation
		gl_TexCoord[1] = lMVP * Pos; 
	
		// check if vertex faces away from light					
		vec3 nv =  vec3( mMVIT * vec4(Pos.xyz, 0.) );		
		float a = acos( dot( normalize(nv), vec3(0.,0.,-1.) ) );	
		if( abs(a) <= PI/2.0 + 0.2 )								
		visible = 0.;										
		else														
		visible = 1.;   	
	}
}